#if !defined ARRAY_H
#define ARRAY_H

#include <cassert>

#include <iostream>
using namespace std;

//#define ARRAY_DEBUG


/* The ArrayBack is where the real storage takes place
	The Array object just references it (it can have
	multiple references if necessary)
 */

template <class T>
class ArrayBack
{

public:
	const unsigned int 		length;

private:		
	T * 				 	data;
	unsigned int			refc;

public:
	ArrayBack(unsigned int length)
		: length(length), data(new T[length]), refc(1) 
		// account for our creator by setting refc to 1
	{
		#ifdef ARRAY_DEBUG
			cout << this << " ArrayBack (" << length << ") created" << endl;
		#endif
	}

	virtual ~ArrayBack()
	{
		#ifdef ARRAY_DEBUG
			cout << this << "ArrayBack destroyed" << endl;
		#endif
		assert(refc == 0);
		assert(data != NULL);
		
		delete [] data; 
		data = NULL; 
	}

	T & operator [](unsigned int i) const
	{
		assert(	data 	!= 	NULL);
		assert( i 		< 	length);

		return data[i];
	}
	
	void operator++(int postfix)
	{
		refc++;
	}
	
	void operator--(int postfix)
	{
		refc--;
		if (refc == 0) delete this;
	}
};




template <class T>
class Array
{

protected:
	ArrayBack<T> *	ab;

public:
	/* Default constructor, constructs an array that
	   MUST later be assigned to */

	Array()
		: ab(NULL)
	{
		#ifdef ARRAY_DEBUG
			cout << this << " Array() created" << endl;
		#endif
	}

	/* Real constructor */

	Array(unsigned int length)
		: ab(new ArrayBack<T>(length))
	{
		#ifdef ARRAY_DEBUG
			cout << this << " Array(" << length << ") created" << endl;
		#endif
	}
	

	/* When a local array needs to be passed in from a parameter,
	   we use this construction. The parameter array will be useless
	   afterwards!
	 */

	Array(const Array & a) : ab(a.ab)
	{
		cout << this << " Array(" << a << ") created" << endl;
		if (ab != NULL) (*ab)++;
	}
	

	void operator = (const Array & a)
	{
		if (ab != NULL) (*ab)--;
		
		ab = a.ab;		// copy backening of a
		
		if (ab != NULL) (*ab)++;
	}


	virtual ~Array()
	{
		#ifdef ARRAY_DEBUG
			cout << this << " Array destroyed" << endl;
		#endif
		if (ab != NULL) (*ab)--;
	}

	T & operator [](unsigned int i) const
	{
		assert(	ab != NULL);

		return (*ab)[i];
	}

	T & operator ()(unsigned int i) const
	{
		assert(	ab != NULL);

		return (*ab)[i];
	}

	unsigned int operator *() const
	{
		assert(	ab != NULL);
		return ab->length;
	}
	
};

template<class T>                            
ostream & operator <<(ostream & s, const Array<T> & a)
{
	s << "[";
	for (unsigned int i = 0; i < *a; i++)
	{
		if (i > 0) s << ", ";
		s << a[i];
	}
	return s << "]";
}





#endif // !defined ARRAY_H
