#include "Explosion.h"
#include "GL/OpenGL.h"
#include "GL/Material.h"
#include "GL/HomLinAlg.h"
#include "GL/Lamp.h"


Explosion::Explosion(const Vector<GLfloat> & x, double max_radius, double speed  )
	: 	x(x), radius(0), flash(NULL), max_radius(max_radius), speed(speed)
{
	assert(*x == 3);
}

Explosion::~Explosion()
{
	if (flash != NULL) delete flash;
}


void Explosion::interactWithObject		(WorldObject 	& other)
{
	return other.interactWithExplosion(*this);
}

void Explosion::interactWithGrid		(Grid 			& other)
{
	return ((WorldObject &)other).interactWithExplosion(*this);
}

void Explosion::interactWithTank		(Tank 			& other)
{
	return ((WorldObject &)other).interactWithExplosion(*this);	
}

void	Explosion::advance(double time, double dt)
{
	radius += dt*speed;

	if (flash == NULL && OpenGL::lightbuffer.available())
	{
		flash = new Lamp(
			Vector<GLfloat>( 	1.0, 	1.0, 	1.0, 	1.0),
			Vector<GLfloat>(	1.0, 	1.0, 	1.0, 	1.0),
			Vector<GLfloat>( 	1.0, 	1.0, 	1.0, 	1.0),
			Vector<GLfloat>(  	x,   					1.0), //positional
			1,
			0.1,
			0.01,
			0,
			Vector<GLfloat>(0.0, 0.0, 0.0),	// equipotent (180 deg. spotlight)
			180
		);
	}
}

bool	Explosion::recollect(vector<WorldObject*> & v)
{
		return radius < max_radius;
}



void	Explosion::setLights(bool lit) const
{
	if (flash != NULL) flash->set(lit);
}

void	Explosion::drawTransparent(const Frustum & f) const
{
	OpenGL::PushMatrix();
	OpenGL::MultMatrix(HomMatrix<GLfloat>::translation(x));

	Material sphereMaterial(
	 Vector<GLfloat>(0, 0, 0, 0),
	 Vector<GLfloat>(1.0, 1.0, 0.9, 1-radius/max_radius),
	 Vector<GLfloat>(1.0, 0.0, 0.0, 1-radius/max_radius),
	 Vector<GLfloat>(0.5, 1.0, 0.0, 1-radius/max_radius)/2,
	 2);

	 sphereMaterial.set();
	 OpenGL::SolidSphere(radius, 20, 20);
	 OpenGL::PopMatrix();
}

const Vector<GLfloat> & Explosion::getX() const
{
	return x;
}

double 					Explosion::getRadius() const
{
	return radius;
}
