#include "FireCone.h"
#include "GL/OpenGL.h"
#include "GL/Material.h"
#include "GL/HomLinAlg.h"
#include "GL/Lamp.h"


FireCone::FireCone(const Vector<GLfloat> & x, const Vector<GLfloat> & d, double max_radius, double speed  )
	: 	x(x), d(d), radius(0), flash(NULL), max_radius(max_radius), speed(speed)
{
}

FireCone::~FireCone()
{
	if (flash != NULL) delete flash;
}


void FireCone::interactWithObject		(WorldObject 	& other)
{
}

void	FireCone::advance(double time, double dt)
{
	radius += dt*speed;

	if (flash == NULL && OpenGL::lightbuffer.available())
	{
		flash = new Lamp(
			Vector<GLfloat>( 	0.0, 	1.0, 	0.0, 	1.0),
			Vector<GLfloat>(	5.0, 	5.0, 	0.0, 	1.0),
			Vector<GLfloat>( 	1.0, 	0.0, 	0.0, 	1.0),
			Vector<GLfloat>(  	x,   					1.0), //positional
			1,
			0.1,
			0.01,
			1,
			d,
			30
		);
	}
}

bool	FireCone::recollect(vector<WorldObject*> & v)
{
	return radius < max_radius;
}



void	FireCone::setLights(bool lit) const
{
	if (flash != NULL) flash->set(lit);
}

void	FireCone::drawTransparent(const Frustum & f) const
{
/*	OpenGL::PushMatrix();
	OpenGL::MultMatrix(	  HomMatrix<GLfloat>::translation(x)
//						* HomMatrix<GLfloat>::translation(Vector<GLfloat>(0,0,radius))
//						* HomMatrix<GLfloat>::rotation(g2r(-90), Vector<GLfloat>(1,0,0))
						);

	Material material(
	 Vector<GLfloat>(0, 0, 0, 0),
	 Vector<GLfloat>(1.0, 1.0, 0.9, 1-radius/max_radius),
	 Vector<GLfloat>(1.0, 0.0, 0.0, 1-radius/max_radius),
	 Vector<GLfloat>(0.5, 1.0, 0.0, 1-radius/max_radius)/2,
	 2);

	 material.set();
	 OpenGL::SolidCone(radius, radius, 20, 20);
	 OpenGL::PopMatrix();
*/
}

const Vector<GLfloat> & FireCone::getX() const
{
	return x;
}

double 					FireCone::getRadius() const
{
	return radius;
}
