#include "Box.h"
#include "OpenGL.h"

Box::Box(const double p[3][2])
{
	for (unsigned int i = 0; i < sizein(this->p); 	i++)
	for (unsigned int j = 0; j < sizein(this->p[i]); 	j++)
	{
		this->p[i][j] = p[i][j];
	}

	for (unsigned int i = 0; i < sizein(this->p); 	i++)
		assert(this->p[i][0] <= this->p[i][1]);
	
}

Box::Box(double x0, double x1, double y0, double y1, double z0, double z1)
{
	p[0][0] = x0;	p[0][1] = x1;
	p[1][0] = y0;	p[1][1] = y1;
	p[2][0] = z0;	p[2][1] = z1;

	for (unsigned int i = 0; i < sizein(this->p); 	i++)
		assert(this->p[i][0] <= this->p[i][1]);
	
}

Box::~Box()
{
}


void Box::draw(const Vector<GLfloat> & color) const
{
	assert(*color == 3);
	OpenGL::PushMatrix();

	GLfloat T[] =
	{
		p[0][1]-p[0][0], 	0.0, 		0.0, 	(p[0][0]+p[0][1])/2,
		0.0, 	p[1][1]-p[1][0], 	0.0, 		(p[1][0]+p[1][1])/2,
		0.0, 	0.0, 		p[2][1]-p[2][0], 	(p[2][0]+p[2][1])/2,
		0.0, 	0.0, 		0.0, 	1.0,
	};
	OpenGL::MultMatrix(Matrix<GLfloat>(4,4,T));
	OpenGL::Disable(GL_LIGHTING);
	OpenGL::Color(color(0), color(1), color(2));
	OpenGL::WireCube(1.0);
	OpenGL::Enable(GL_LIGHTING);

	OpenGL::PopMatrix();
}


void Box::Union(Box *& b1, const Box * b2)
{
	// an empty partner doesn't change us
	if (b2 == NULL) return;

	// if we were empty, make us b2
	if (b1 == NULL)
	{
		b1 = new Box(b2->p);
	}
	else // the real min & max thing
	{
		b1 = new Box(
			w_min(b1->p[0][0], b2->p[0][0]), w_max(b1->p[0][1], b2->p[0][1]),
			w_min(b1->p[1][0], b2->p[1][0]), w_max(b1->p[1][1], b2->p[1][1]),
			w_min(b1->p[2][0], b2->p[2][0]), w_max(b1->p[2][1], b2->p[2][1]));
	}
}
