#include "OpenGL.h"
#include <cstdio>
#include "utils.h"
#include <cstdlib>
#include <cassert>


#define CHECK() 		check(__PRETTY_FUNCTION__)
#define CHECKGLU() 		CHECK()
#define CHECKGLUT() 	CHECK()

	

LightBuffer OpenGL::lightbuffer;
bool 		OpenGL::inBeginEndBlock = false;
	

void OpenGL::check(const char * caller)
{
	if (inBeginEndBlock) return;
	GLenum err = glGetError();
	
	switch (err)
	{
	case GL_NO_ERROR: 
	return;

	case GL_INVALID_ENUM:
		fprintf(stderr, "%s: GL_INVALID_ENUM: An unacceptable value is specified for an enumerated argument.  \n", caller);
	break;

	case GL_INVALID_VALUE:
		fprintf(stderr, "%s: GL_INVALID_VALUE: A  numeric  argument is out of range.  \n", caller);
	break;

	case GL_INVALID_OPERATION:
		fprintf(stderr, "%s: GL_INVALID_OPERATION:  The specified operation is not allowed in the current state.  \n", caller);
	break;
	
	case GL_STACK_OVERFLOW:
		fprintf(stderr, "%s: GL_STACK_OVERFLOW:  This  command  would cause a stack overflow.  \n", caller);
	break;
	
	case GL_STACK_UNDERFLOW:
		fprintf(stderr, "%s: GL_STACK_UNDERFLOW: This command would cause a stack underflow.  \n", caller);
	break;
	
	case GL_OUT_OF_MEMORY:
		fprintf(stderr, "%s: GL_OUT_OF_MEMORY: There  is  not  enough  memory left to execute the command.  The state of the GL is undefined, except for the state of the error flags, after this error is recorded.\n", caller);
	break;

	case GL_TABLE_TOO_LARGE:
		fprintf(stderr, "%s: GL_TABLE_TOO_LARGE: The specified table exceeds the implementation's maximum supported table size.\n", caller);
	break;
	
	default:
		fprintf(stderr, "%s: UNKNOWN ERROR\n", caller);
	break;
	
	}

	assert(false);	
}






void OpenGL::Begin(GLenum mode)
{
	glBegin(mode);
	inBeginEndBlock = true;
	CHECK();
}

void OpenGL::Clear(GLbitfield mask)
{
	glClear(mask);
	CHECK();
}

void OpenGL::ClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
{
	glClearColor(red, green, blue, alpha);
	CHECK();
}

void OpenGL::Color(GLfloat red, GLfloat green, GLfloat blue)
{
	glColor3f(red, green, blue);
	CHECK();
}

void OpenGL::Color(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha)
{
	glColor4f(red, green, blue, alpha);
	CHECK();
}

void OpenGL::CullFace(GLenum mode)
{
	glCullFace(mode);
	CHECK();
}

void OpenGL::Disable(GLenum cap)
{
	glDisable(cap);
	CHECK();
}

void OpenGL::Enable(GLenum cap)
{
	glEnable(cap);
	CHECK();
}

void OpenGL::End()
{
	glEnd();
	inBeginEndBlock = false;
	CHECK();
	
}

void OpenGL::Lightf(GLenum light, GLenum pname, GLfloat param)
{
	glLightf(light, pname, param);
	CHECK();
}

void OpenGL::Lightfv(GLenum light, GLenum pname, const GLfloat *params)
{
	glLightfv(light, pname, params);
	CHECK();
}

void OpenGL::LoadIdentity()
{
	glLoadIdentity();
	CHECK();
}

void OpenGL::Materialf(GLenum face, GLenum pname, GLfloat param)
{
	glMaterialf(face, pname, param);
	CHECK();
}

void OpenGL::Materialfv(GLenum face, GLenum pname, const GLfloat *params)
{
	glMaterialfv(face, pname, params);
	CHECK();
}

void OpenGL::MatrixMode( GLenum mode)
{
	glMatrixMode(mode);
	CHECK();
}



void OpenGL::PolygonMode(GLenum face, GLenum mode)
{
	glPolygonMode(face, mode);
	CHECK();
}





void OpenGL::Viewport(GLint x, GLint y, GLsizei width, GLsizei height)
{
	glViewport(x, y, width, height);
	CHECK();
}

void OpenGL::Perspective(GLdouble fovy, GLdouble aspect, GLdouble
	 zNear, GLdouble zFar)
{
	gluPerspective(fovy, aspect, zNear, zFar);
	CHECKGLU();
}

void OpenGL::CreateWindow(const char *name)
{
	glutCreateWindow(name);
	CHECKGLUT();

	// now we have the OpenGL context !
	GLint lights = -1;
	glGetIntegerv(GL_MAX_LIGHTS, &lights);
	CHECK();
	lightbuffer.resize(lights);
	
}

void OpenGL::DisplayFunc(void (*func)(void))
{
	glutDisplayFunc(func);
	CHECKGLUT();
}

void OpenGL::IdleFunc(void (*func)(void))
{
	glutIdleFunc(func);
	CHECKGLUT();
}

void OpenGL::Init(int *argcp, char **argv)
{
	glutInit(argcp, argv);
	CHECKGLUT();
}

void OpenGL::InitDisplayMode(unsigned int mode)
{
	glutInitDisplayMode(mode);
	CHECKGLUT();
}

void OpenGL::InitWindowPosition(int x, int y)
{
	glutInitWindowPosition(x,y);
	CHECKGLUT();
}

void OpenGL::InitWindowSize(int width, int height)
{
	glutInitWindowSize(width, height);
	CHECKGLUT();
}

void OpenGL::MainLoop()
{
	glutMainLoop();
	CHECKGLUT();
}

void OpenGL::PostRedisplay()
{
	glutPostRedisplay();
	CHECKGLUT();
}

void OpenGL::ReshapeFunc(void (*func)(int width, int height))
{
	glutReshapeFunc(func);
	CHECKGLUT();
}

void OpenGL::SwapBuffers()
{
	glutSwapBuffers();
	CHECKGLUT();
}

void OpenGL::VertexPointer( GLint size, GLenum type, GLsizei stride, const GLvoid *ptr )
{
	glVertexPointer(size, type, stride, ptr);
	CHECK();
}


void OpenGL::NormalPointer(GLenum type, GLsizei stride, const GLvoid *ptr )
{
	glNormalPointer(type, stride, ptr);
	CHECK();
}


void OpenGL::ArrayElement( GLint i )
{
	glArrayElement( i );
	CHECK();
}


void OpenGL::EnableClientState( GLenum cap)
{
	glEnableClientState( cap);
	CHECK();
}

void OpenGL::DisableClientState( GLenum cap)
{
	glEnableClientState( cap);
	CHECK();
}

void OpenGL::DrawElements( GLenum mode, GLsizei count, GLenum type, const GLvoid *indices )
{
	glDrawElements( mode, count, type, indices );
	CHECK();
}



void OpenGL::PushMatrix()
{
	glPushMatrix();
	CHECK();

}

void OpenGL::PopMatrix()
{
	glPopMatrix();
	CHECK();
}

GLuint OpenGL::GenLists(GLsizei range)
{
	GLuint result = glGenLists(range);
	CHECK();
	return result;
}

void   OpenGL::CallList(GLuint list)
{
	glCallList(list);
	CHECK();
}

void   OpenGL::DeleteLists(GLuint list, GLsizei range)
{
	glDeleteLists(list, range);
	CHECK();
}

void   OpenGL::NewList( GLuint list, GLenum mode)
{
	glNewList(list, mode);
	CHECK();
}

void 	OpenGL::EndList(void)
{
	glEndList();
	CHECK();
}


void 	OpenGL::KeyboardFunc(void (*func)(unsigned char key, int x, int y))
{
	glutKeyboardFunc(func);
	CHECKGLUT();
}

void 	OpenGL::KeyboardUpFunc(void (*func)(unsigned char key, int x, int y))
{
	glutKeyboardUpFunc(func);
	CHECKGLUT();
}


void 	OpenGL::SpecialFunc(void (*func)(int key, int x, int y))
{
	glutSpecialFunc(func);
	CHECKGLUT();	
}


void 	OpenGL::SpecialUpFunc(void (*func)(int key, int x, int y))
{
	glutSpecialUpFunc(func);
	CHECKGLUT();	
}

void 	OpenGL::IgnoreKeyRepeat(bool ignore)
{
	glutIgnoreKeyRepeat(ignore);
	CHECKGLUT();	
}

int 	OpenGL::GetModifiers()
{
	int result = glutGetModifiers();
	CHECKGLUT();
	return result;	
}



void OpenGL::MultMatrix( const Matrix<GLfloat> & m)
{
	assert(m.nRows() == 4);
	assert(m.nCols() == 4);
	
	glMultMatrixf(m);
	CHECK();
}

void OpenGL::MultMatrix( const Matrix<GLdouble> & m)
{
	assert(m.nRows() == 4);
	assert(m.nCols() == 4);

	glMultMatrixd(m);
	CHECK();
}

void OpenGL::Normal(const Vector<GLfloat> & v)
{
	assert(*v == 3);
	
	glNormal3fv(v);
	CHECK();
}

void OpenGL::Normal(const Vector<GLdouble> & v)
{
	assert(*v == 3);
	
	glNormal3dv(v);
	CHECK();
}


void OpenGL::Vertex(const Vector<GLfloat> & v)
{
	switch (*v)
	{
		case 2: glVertex2fv(v); break;
		case 3: glVertex3fv(v); break;
		case 4: glVertex4fv(v); break;
		default: assert(false);
	}	
	CHECK();
}

void OpenGL::Vertex(const Vector<GLdouble> & v)
{
	switch (*v)
	{
		case 2: glVertex2dv(v); break;
		case 3: glVertex3dv(v); break;
		case 4: glVertex4dv(v); break;
		default: assert(false);
	}	
	CHECK();

}


void 	OpenGL::BlendFunc(GLenum sfactor, GLenum dfactor)
{
	glBlendFunc(sfactor, dfactor);
	CHECK();
}

void 	OpenGL::SolidSphere(GLdouble radius, GLint slices, GLint stacks)
{
	glutSolidSphere(radius, slices, stacks);
	CHECKGLUT();
}

void 	OpenGL::WireSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutWireSphere(radius, slices, stacks);
	CHECKGLUT();
}

void 	OpenGL::SolidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutSolidCone(base, height, slices, stacks);
	CHECKGLUT();
}

void	OpenGL::WireCone(GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutWireCone(base, height, slices, stacks);
	CHECKGLUT();
}

void 	OpenGL::SolidCube(GLdouble size)
{
	glutSolidCube(size);
	CHECKGLUT();
}

void	OpenGL::WireCube(GLdouble size)
{
	glutWireCube(size);
	CHECKGLUT();
}



void 	OpenGL::DepthMask( GLboolean flag )
{
	glDepthMask(flag);
	CHECK();
}


Matrix<GLfloat> OpenGL::GetModelViewMatrix()
{
	float MV[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, MV);
	CHECK();
	return Matrix<GLfloat>(4, 4, MV, false);
}

Matrix<GLfloat> OpenGL::GetProjectionMatrix()
{
	float P[16];
	glGetFloatv(GL_PROJECTION_MATRIX, P);
	CHECK();
	return Matrix<GLfloat>(4, 4, P, false);
}




