#ifndef __OPEN_H
#define __OPEN_H

#include <GL/glut.h>
#include "LinAlg.h"
#include "LightBuffer.h"


class OpenGL
{
private:
	static void check(const char * caller);
	static bool inBeginEndBlock;
	OpenGL();	// prevent instantiation

public:
	static LightBuffer	 lightbuffer;

	 static void Begin(GLenum mode);
	 static void Clear(GLbitfield mask);
	 static void ClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
	 
	 
	 static void Color(GLfloat red, GLfloat green, GLfloat blue);
	 static void Color(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);	 
	 static void CullFace(GLenum mode);
	 static void Disable(GLenum cap);
	 static void Enable(GLenum cap);
	 static void End();
	 static void Lightf(GLenum light, GLenum pname, GLfloat param);
	 static void Lightfv(GLenum light, GLenum pname, const GLfloat *params);
	 static void LoadIdentity();
	 static void Materialf(GLenum face, GLenum pname, GLfloat param);
	 static void Materialfv(GLenum face, GLenum pname, const GLfloat *params);
	 static void MatrixMode( GLenum mode);
	 static void PolygonMode( GLenum face, GLenum mode);
	 	 
	 static void MultMatrix( const Matrix<GLfloat> & m);
	 static void MultMatrix( const Matrix<GLdouble> & m);
	 static void Normal(const Vector<GLfloat> & v);
	 static void Normal(const Vector<GLdouble> & v);
	 static void Vertex(const Vector<GLfloat> & v);	
	 static void Vertex(const Vector<GLdouble> & v);

	 static Matrix<GLfloat> GetModelViewMatrix();
	 static Matrix<GLfloat> GetProjectionMatrix();
	 
	 
	 static void Viewport( GLint x, GLint y, GLsizei width, GLsizei height);
	 static void Perspective(GLdouble fovy, GLdouble aspect, GLdouble
	 zNear, GLdouble zFar);
	 static void CreateWindow(const char *name);
	 static void DisplayFunc(void (*func)(void));
	 static void IdleFunc(void (*func)(void));
	 static void Init(int *argcp, char **argv);
	 static void InitDisplayMode(unsigned int mode);
	 static void InitWindowPosition(int x, int y);
	 static void InitWindowSize(int width, int height);
	 static void MainLoop();
	 static void PostRedisplay();
	 static void ReshapeFunc(void (*func)(int width, int height));
	 static void SwapBuffers();
	 
	 static void VertexPointer( GLint size, GLenum type, GLsizei stride, const GLvoid *ptr );
	 static void NormalPointer( GLenum type, GLsizei stride, const GLvoid *ptr );
	 static void ArrayElement( GLint i );
	 static void EnableClientState( GLenum cap);
	 static void DisableClientState( GLenum cap);
	 static void DrawElements( GLenum mode, GLsizei count, GLenum type, const GLvoid *indices );
     static void PushMatrix();
     static void PopMatrix();

     static GLuint	GenLists(GLsizei range);
     static void	DeleteLists(GLuint list, GLsizei range);
     static void	CallList(GLuint list);

     static void	NewList(GLuint list, GLenum mode);
     static void 	EndList(void);
	 
	 
	static	void 	KeyboardFunc(void (*func)(unsigned char key, int x, int y));
	static	void 	KeyboardUpFunc(void (*func)(unsigned char key, int x, int y));
 	static 	void 	SpecialFunc(void (*func)(int key, int x, int y));
 	static 	void 	SpecialUpFunc(void (*func)(int key, int x, int y));
	static 	void 	IgnoreKeyRepeat(bool ignore);
	static 	int 	GetModifiers();
	
	static void 	BlendFunc(GLenum sfactor, GLenum dfactor);

 	static	void 	SolidSphere(GLdouble radius, GLint slices, GLint stacks);
    static 	void 	WireSphere (GLdouble radius, GLint slices, GLint stacks);
    static 	void 	SolidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks);
    static	void	WireCone(GLdouble base, GLdouble height, GLint slices, GLint stacks);
    static 	void 	SolidCube(GLdouble size);
    static	void	WireCube(GLdouble size);

    static 	void 	DepthMask( GLboolean flag );

};





#endif //__OPEN_H
