#include "SP.h"
#include "Box.h"
#include "Sphere.h"
#include <cassert>
#include "Frustum.h"
#include "LinAlg.h"

SP::SP()
	: bound(NULL), boundSphere(NULL), parent(NULL)
{
}

SP::~SP()
{
	if (bound != NULL)
	{
		delete bound;
		delete boundSphere;
		bound = NULL;
		boundSphere = NULL;
	}	
}

void SP::invalidate()
{
	if (bound != NULL)
	{
		delete bound;
		delete boundSphere;
		bound = NULL;
		boundSphere = NULL;
	}
	if (parent != NULL)
	{
		parent->invalidate();
	}	
}

Box *	SP::getBound()
{
	if (bound == NULL) 
	{
		bound = recalcBound();
		assert(bound != NULL);
		boundSphere = new Sphere(*bound);
	}
	return bound;
}


SPNode::SPNode()
{
}

SPNode::~SPNode()
{
	for (vector<SP*>::iterator it = children.begin(); it != children.end(); it++)
		delete (*it);
}

void SPNode::draw()
{
	for (vector<SP*>::iterator it = children.begin(); it != children.end(); it++)
		(*it)->draw();
}

void SPNode::check(const Frustum & f, bool drawBound)
{
	getBound();
	
	/* start with loose check wigh sphere */
	switch(f.getTopoRel(*boundSphere))
	{
		case OUTSIDE: 	/* nothing to do for entire subtree */
			if (drawBound) boundSphere->draw(Vector<GLfloat>(1,0,0));
		break;

		case INSIDE:	/* even a too wide bound is entirely on screen */
			if (drawBound) bound->draw(Vector<GLfloat>(1,1,1));
			draw();
		break;
		
		/* it seems to depend... lets check with the children */
		case INTERSECT:		
			switch(f.getTopoRel(*bound))
			{
				case OUTSIDE: 	/* nothing to do for entire subtree */
					if (drawBound) bound->draw(Vector<GLfloat>(1,0,0));
				break;

				case INSIDE : 	/* no more questions */
					if (drawBound) bound->draw(Vector<GLfloat>(1,1,1));
					draw();
				break;

				case INTERSECT: /* gather the pieces (recurse) */
//					if (drawBound) bound->draw(Vector<GLfloat>(1,1,0));
					for (vector<SP*>::iterator it = children.begin(); it != children.end(); it++)
						(*it)->check(f, drawBound);
				break;
			}
		break;
	}
}

Box *	SPNode::recalcBound()
{
	assert(bound == NULL);
	assert(children.size() > 0);

	for (vector<SP*>::iterator it = children.begin(); it != children.end(); it++)
	{
		Box::Union(bound, (*it)->getBound());
	}
	assert(bound != NULL);
	return bound;
}



void	SPNode::addChild(SP * child)
{
	assert(child->parent == NULL);

	children.push_back(child);
	child->parent = this;
}


SPLeaf::SPLeaf()
{
}

SPLeaf::~SPLeaf()
{
}

void SPLeaf::check(const Frustum & f, bool drawBound)
{
	getBound();
	switch(f.getTopoRel(*boundSphere))
	{
			case OUTSIDE:
				if (drawBound) bound->draw(Vector<GLfloat>(1,0,0));
			break;
			
			case INSIDE:
				if (drawBound) bound->draw(Vector<GLfloat>(1,1,1));
				draw();
			break;
			
			case INTERSECT:
				if (drawBound) bound->draw(Vector<GLfloat>(1,1,0));
				draw();
			break;
	}
}
