#include "Sphere.h"
#include "OpenGL.h"
#include "HomLinAlg.h"
#include "utils.h"

#include "Box.h"


const double Sphere::sqrt3p2 = sqrt(3.0)/2.0;

Sphere::Sphere(const Vector<GLfloat> & x, double radius)
	:x(x), radius(radius)
{
	assert(radius >= 0);
	assert(*x == 4);
	assert(x(3) == 1.0);
}


Sphere::Sphere(const Box & b)
	: 	x(
			Vector<GLfloat>( 	
			(b.p[0][0]+b.p[0][1])/2,
			(b.p[1][0]+b.p[1][1])/2,
			(b.p[2][0]+b.p[2][1])/2,
			1.0)
		), 
		radius(
			sqrt3p2 * w_max(b.p[0][1]-b.p[0][0], 
			w_max(b.p[1][1]-b.p[1][0], 
			  	  b.p[2][1]-b.p[2][0]))
		)
{

}

Sphere::~Sphere()
{
}


void Sphere::draw(const Vector<GLfloat> & color) const
{
	OpenGL::PushMatrix();
	OpenGL::MultMatrix(HomMatrix<GLfloat>::translation(Vector<GLfloat>(3,x)));
	OpenGL::Disable(GL_LIGHTING);
	OpenGL::Color(color(0),color(1),color(2));
	OpenGL::WireSphere(radius, 10, 10);
	OpenGL::Enable(GL_LIGHTING);	
	OpenGL::PopMatrix();

}
