#include "GL/HomLinAlg.h"
#include "GL/Material.h"
#include "GL/GLDisplayList.h"

#include "Grid.h"
#include "utils.h"
#include "array.h"
#include <time.h>
#include "Tank.h"
#include "Explosion.h"



Grid::Grid(int logsize) : model(logsize), dl(NULL)
{
}

Grid::~Grid()
{
	if (dl != NULL) delete dl;
}              
               
double Grid::getH(const Vector<GLfloat> & xz) const
{
	return model.getH(xz);
}


void Grid::draw(const Frustum & f) const
{
	model.draw(f, drawBounds);
}

void Grid::interactWithObject	(WorldObject 	& other)
{
	other.interactWithGrid(*this);
}

void Grid::interactWithPhysicalObject(PhysicalObject & other)
{
	Vector<GLfloat> g(0,-9.8,0,0,0,0);
	// first, we handle gravity
	other.accellerate(g);

	// then we handle repulsion by the ground

	const Vector<GLfloat> & x = other.getX();
	double d = x(1) - getH(Vector<GLfloat>(x(0), x(2)));
//	cout << "Distance to ground: " << d << endl;
	d -= 1.5;

	if (d < 0) other.onHitGround(d);
}

void Grid::interactWithExplosion		(Explosion		& other)
{
	model.subtractSphere(other.getX(), other.getRadius());
	if (dl != NULL) { delete dl; dl = NULL; }
}

void Grid::advance(double time, double dt)          
{
		// grids lay still
}


void Grid::setDrawBounds(bool value)
{
	drawBounds = value;
}
