#ifndef _GRIDMODEL_H
#define _GRIDMODEL_H



#include "GL/OpenGL.h"
#include "GL/Material.h"
#include "GL/LinAlg.h"
#include "GL/GLModel.h"

class GridPatch;
class SPNode;
class Frustum;

class GridModel
{
public:
			GridModel(int logsize);
	virtual ~GridModel();
	
	void drawVertexNormals(double length) const;
	void drawFaceNormals(double length) const;
	
	void draw(const Frustum & f, bool drawBounds = false) const;

	double getH(const Vector<GLfloat> & xz) const;

	void subtractSphere(const Vector<GLfloat> & p, double radius);

	
protected:
	inline unsigned int 	improve(int x) const;
	
	template<class T>
	inline T get(T ** h, int x, int y) const;
	
	template<class T>
	inline void put(T ** h, int x, int y, T v);
	
	
	double mod(double s);

	void 	generate();	
	void 	smooth(unsigned int x0, unsigned int y0, unsigned int sx, unsigned int sy);	
	void 	patch();
	void 	partition(SPNode * sp, unsigned int x0, unsigned int y0, unsigned int sx, unsigned int sy);
	void 	computeFaceNormals(unsigned int x0, unsigned int x1, unsigned int y0, unsigned int y1);
	void 	computeVertexNormals(unsigned int x0, unsigned int x1, unsigned int y0, unsigned int y1);
	void 	bufferize(unsigned int x0, unsigned int x1, unsigned int y0, unsigned int y1);


private:
	double **					h;
	Vector<GLfloat> **			vns;
	Vector<GLfloat> ***			normals;
	unsigned int 				s;
	GLfloat *					vertexbuffer;
	GLfloat *					normalbuffer;

	static Material 			GridMaterial;
	static const int 			ns[2][3][2];
	static const int 			fn[6][4];

	unsigned int patchesx;
	unsigned int patchesy;

	GridPatch ***				patches;
	SPNode *					sp;

	friend class GridPatch;
};


#endif //_GRIDMODEL_H
