#include "PhysicalObject.h"
#include "GL/OpenGL.h"
#include "GL/HomLinAlg.h"

PhysicalObject::PhysicalObject() : x(6), v(6), a(6), m(10)
{
}

	
PhysicalObject::~PhysicalObject()
{
}

void PhysicalObject::draw(const Frustum & f) const
{
	OpenGL::PushMatrix();
	OpenGL::MultMatrix(getMatrix());

	drawPO(f);

	OpenGL::PopMatrix();
}

Matrix<GLfloat> PhysicalObject::getMatrix() const
{
	//remember the order DOES matter!!!
	// roll-pitch-yaw is an OBJECT (not WORLD) convention, so we have to post-multiply (= pre-write)
	return	  HomMatrix<GLfloat>::translation(Vector<GLfloat>(x(0), x(1), x(2)))
			* HomMatrix<GLfloat>::rotation(x(3), Vector<GLfloat>(0,0,1))  //roll
			* HomMatrix<GLfloat>::rotation(x(4), Vector<GLfloat>(1,0,0))  //pitch
			* HomMatrix<GLfloat>::rotation(x(5), Vector<GLfloat>(0,1,0)); //yaw	
}


void PhysicalObject::advance(double time, double dt)
{
	v = v + a * dt;           
	x = x + v * dt;

	a = Vector<GLfloat>(6); // reset to zero
}

const Vector<GLfloat> &  	PhysicalObject::getX() const { return x; }
const Vector<GLfloat> & 	PhysicalObject::getV() const { return v; }         
const Vector<GLfloat> & 	PhysicalObject::getA() const { return a; }         
double 						PhysicalObject::getM() const { return m; }

void 	PhysicalObject::setX(const Vector<GLfloat> & x) { this->x = x; }
void 	PhysicalObject::setV(const Vector<GLfloat> & v) { this->v = v; }
void	PhysicalObject::setA(const Vector<GLfloat> & a) { this->a = a; }


void PhysicalObject::accellerate(const Vector<GLfloat> & a)
{
		this->a += a;
}


void PhysicalObject::interactWithObject	(WorldObject 	& other)
{
	other.interactWithPhysicalObject(*this);
}

void PhysicalObject::interactWithGrid	(Grid 			& other)
{
	// they handle us
	((WorldObject&)other).interactWithPhysicalObject(*this);
}
