#ifndef _PHYSICAL_OBJECT_H
#define _PHYSICAL_OBJECT_H

#include "GL/OpenGL.h"
#include "GL/LinAlg.h"
#include "WorldObject.h"


/* x, v, a are all three 6 dimensional vectors

the first three coefficients are translational
position (m),
velocity (m/s) and
acceleration (m/s^2)

the second three coefficients are rotational
angle (rad)
angular velocity (rad/s)
angular acceleration(rad/s^2)

*/

class Grid;

class PhysicalObject  : public WorldObject
{
public:
			PhysicalObject();
	virtual ~PhysicalObject();

	virtual void draw(const Frustum & f) const;
	virtual void drawPO(const Frustum & f) const = 0;
	virtual void advance(double time, double dt);

	virtual void onHitGround(double distance) = 0;

	const Vector<GLfloat> & 	getX() const;
	const Vector<GLfloat> & 	getV() const;
	const Vector<GLfloat> & 	getA() const;
	double 						getM() const;
	Matrix<GLfloat> 			getMatrix() const;

	void 	setX(const Vector<GLfloat> & x);
	void 	setV(const Vector<GLfloat> & v);
	void	setA(const Vector<GLfloat> & a);
	

	void accellerate(const Vector<GLfloat> & a);
	
protected:
	void interactWithObject	(WorldObject 	& other);
	void interactWithGrid	(Grid 			& other);
	
	Vector<GLfloat>	x;
	Vector<GLfloat>	v;
	Vector<GLfloat>	a;
	double 			m;
};




#endif // _PHYSICAL_OBJECT_H
