#include "GL/OpenGL.h"
#include "GL/HomLinAlg.h"
#include "GL/Material.h"

#include "Tank.h"
#include "array.h"
#include "utils.h"
#include "Grid.h"
#include "Bullet.h"
#include "FireCone.h"
#include "Explosion.h"


Tank::Tank()
	: 	a_cabine(0), 	a_funnel(g2r(80)),
		av_cabine(0), 	av_funnel(0),
		drive(0), 		turn(0),
		firing(false), 	firedelay(0.0)
{
	// place'em somewhere
	x = Vector<GLfloat>(2*(double)rand()/RAND_MAX-1, 1, 2*(double)rand()/RAND_MAX-1, 0, 0, 0)*20;
}

	
Tank::~Tank()
{
}




void Tank::onHitGround(double distance)
{
	x -= Vector<GLfloat>(0,distance,0,0,0,0);  // we're below ground, so subtract
	// check wether our velocity is not pointing downward
	double vy = v(1);
	if (vy < 0) v -= Vector<GLfloat>(0,vy,0,0,0,0);

	// check wether our velocity is not pointing downward
	double ay = a(1);
	if (ay < 0) a -= Vector<GLfloat>(0,ay,0,0,0,0);
	
}



void Tank::drawPO(const Frustum & f) const
{
	model.draw(a_cabine, a_funnel);

/*	OpenGL::Enable(GL_BLEND);
	OpenGL::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	Material sphereMaterial(
	 Vector<GLfloat>(0.5, 0.0, 0.0, 0.5),
	 Vector<GLfloat>(0.5, 0.0, 0.0, 0.5),
	 Vector<GLfloat>(0.5, 0.0, 0.0, 0.5),
	 Vector<GLfloat>(0, 0, 0, 0),
	 2);

	 sphereMaterial.set();
	 OpenGL::SolidSphere(2.5, 20, 20);

	 OpenGL::Disable(GL_BLEND);
*/
}


#define CLAMP(X, Min, Max)	X =  X < Min ? Min : X > Max ? Max : X

void Tank::advance(double time, double dt)
{
	PhysicalObject::advance(time, dt);

	firedelay += dt;

	x += Vector<GLfloat>(getMatrix() * Vector<GLfloat>(0.0,0.0,10*drive,0.0), Vector<GLfloat>(0.0f,(float)(g2r(22.5)*turn))) * dt;
	
	a_cabine += av_cabine * dt;
	a_funnel += av_funnel * dt;
	
//	CLAMP(a_cabine, -M_PI/2, M_PI/2);
	CLAMP(a_funnel, g2r(0), g2r(105));

//	a_cabine = 	sin(time)/2;
//	a_funnel = 	(g2r(70)-g2r(85))*(1+sin(5*time)/2) + g2r(85);
}




bool	Tank::recollect(vector<WorldObject*> & v)
{
	if (firing && firedelay > 0.3)
	{
		firedelay = 0;
		
		Matrix<GLfloat> M 		=   getMatrix()
								*	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0, -0.2, -0.3))
								*	HomMatrix<GLfloat>::rotation(a_cabine, Vector<GLfloat>(0, 1, 0))
								*	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0, 0, 0.2))
								*	HomMatrix<GLfloat>::rotation(a_funnel, Vector<GLfloat>(1, 0, 0))
							;
		// now the way to go is in the y direction

		Vector<GLfloat>	z3(0,0,0);

		// end of the funnel position
		Vector<GLfloat> fp = Vector<GLfloat>(3, M * Vector<GLfloat>(0,3, 0,1));

		// fire direction
		#define Pu   (double)rand()/RAND_MAX-0.5
		Vector<GLfloat> fd = Vector<GLfloat>(3, M * Vector<GLfloat>(Pu,20+Pu,Pu,0));

		Bullet * b = new Bullet();
		b->setX(Vector<GLfloat>(fp, z3));
		b->setV(Vector<GLfloat>(fd, z3));
		b->setA(Vector<GLfloat>(0,0,0,0,0,0)); // no accel

		v.push_back(b);
		v.push_back(new FireCone(fp, fd, 0.3, 3.0));
			
	}
		return true;
}


void 	Tank::interactWithExplosion		(Explosion		& other)
{
	double d = (Vector<GLfloat>(3, x) - other.getX()).length();
	if (d < other.getRadius() + 1.0)
	{
		v += Vector<GLfloat>(0,10, 0, 0, 0, 0);
	}
}
