#include <cassert>
#include <iostream>
#include <cstdio>

#include "GL/OpenGL.h"
#include "GL/Lamp.h"
#include "GL/HomLinAlg.h"
#include "GL/Frustum.h"

#include "TankCombat.h"
#include "utils.h"
#include "Tank.h"
#include "Bullet.h"
#include "GL/Lamp.h"
#include "GL/Frustum.h"
#include "Grid.h"
#include "Axes.h"
#include "Water.h"
#include "Tank.h"


#include <iostream>
using namespace std;
                     
#define NTANKS 3

TankCombat::TankCombat(unsigned int gridsize)
: 	model2world(Matrix<GLfloat>::identity(4)),
	vx0(1), vy0(1),  
	vx1(1),  vy1(1),
	gridsize(gridsize), 
	fieldsize(1 << gridsize),
	d(1.7 * fieldsize)
{
	// -y (top) view
	world2view[0][0] = 	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0.0, 0.0, -d))
						* HomMatrix<GLfloat>::rotation(g2r(90), Vector<GLfloat>(1.0, 0.0, 0.0));
	// z (front) view 						
    world2view[0][1] = 	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0.0, 0.0, -d))
						* HomMatrix<GLfloat>::rotation(g2r(0), Vector<GLfloat>(1.0, 0.0, 0.0));
	// x (left) view						
    world2view[1][0] = 	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0.0, 0.0, -d))
						* HomMatrix<GLfloat>::rotation(g2r(90), Vector<GLfloat>(0.0, 1.0, 0.0));
	// nice view						
    world2view[1][1] = 	HomMatrix<GLfloat>::translation(Vector<GLfloat>(0.0, 0.0, -d))
						* HomMatrix<GLfloat>::rotation(g2r(30), Vector<GLfloat>(1.0, 0.0, 0.0));


	axes = 		new Axes(1);	
	grid = 		new Grid(gridsize);
	water = 	new Water(fieldsize*1.5, w_min(16, fieldsize/4), 1000);
	tank = 		new Tank[NTANKS];

	objects.push_back( grid);
	objects.push_back( water);
	for (unsigned int i = 0; i < NTANKS; i++) objects.push_back(&tank[i]);


	// sun
	lamps.push_back(new Lamp(
		Vector<GLfloat>( 	Vector<GLfloat>(0.0, 	0.0, 	0.0)/2, 	1.0),
		Vector<GLfloat>(	Vector<GLfloat>(1.0, 	1.0, 	1.0)/2, 	1.0),
		Vector<GLfloat>( 	Vector<GLfloat>(1.0, 	1.0, 	1.0)/2, 	1.0),
		Vector<GLfloat>(   -1.0,   -1.0,   -1.0))
	);

    Vector<GLfloat> look =		Vector<GLfloat>(0,-0.1,1).normalize();
    
	// beam that follows tank 1 , defined in tank1's coordinate frame !!!
	lamps.push_back(new Lamp(
		Vector<GLfloat>( 	Vector<GLfloat>(0.0, 	0.0, 	0.0), 	1.0),
		Vector<GLfloat>(	Vector<GLfloat>(1.0, 	1.0, 	1.0), 	1.0),
		Vector<GLfloat>( 	Vector<GLfloat>(1.0, 	1.0, 	1.0), 	1.0),
        Vector<GLfloat>(	-look*5, 				1.0), // spot : w =/= 0
        1,
        0.0001,
        0,
        1,
        look,
        22.5)
	);
        
}


TankCombat::~TankCombat()
{
	delete axes;
	delete grid;
	delete water;
	delete [] tank;
}


/*------------------------------------------------------------------------------
GUI Callback functions

These functions record the mouse position and perform a rotation on
mouse drag actions.

*/



void TankCombat::mouseButtonDown(float x, float y) 
{
	Point mouse(x, y);

	printf("mouse button down at (%f, %f)\n", mouse.x, mouse.y);

	previousMouse = Point(x, y);
}

void TankCombat::mouseButtonUp(float x, float y) 
{
	Point mouse(x, y);


	printf("mouse butostream & operator << (ostream & s, const Point & p);ton up at (%f, %f)\n", mouse.x, mouse.y);

	previousMouse = Point(x, y);
}

void TankCombat::mouseDrag(float x, float y)
{
	Point mouse(x, y);
	Point d = mouse-previousMouse;
	
	printf("mouse drag to (%f, %f), (%f, %f)\n", mouse.x, mouse.y, d.x, d.y);
	
	model2world = HomMatrix<GLfloat>::rotation( d.x/50.0, Vector<GLfloat>(0, 1, 0)) *
	HomMatrix<GLfloat>::rotation(-d.y/50, Vector<GLfloat>(1.0, 0, 0)) * model2world;
	
	previousMouse = Point(x, y);
}


#define CABINE_R	g2r(15)


#define FUNNEL_R	g2r(10)

void TankCombat::processNormalKeyDown	(unsigned char key, int modifiers, const Point & mouse)
{
	switch(key)
	{
		case 27:	exit(0); // user pressed escape


		case 13:	tank[0].firing = true; break;
		case 'a':
		case 'A':	tank[0].av_cabine += CABINE_R; break;
		case 'd':
		case 'D':	tank[0].av_cabine -= CABINE_R; break;
		case 'w':
		case 'W':	tank[0].av_funnel -= FUNNEL_R; break;
		case 's':
		case 'S':	tank[0].av_funnel += FUNNEL_R; break;

//		case ' ': sleep(30); break; // screenshot
		
		default:
			 Scene::processNormalKeyDown(key, modifiers, mouse);
	}
}

void TankCombat::processNormalKeyUp	(unsigned char key, int modifiers, const Point & mouse)
{
	switch(key)
	{
		case 13:	tank[0].firing = false; break;
		case 'a':
		case 'A':	tank[0].av_cabine -= CABINE_R; break;
		case 'd':
		case 'D':	tank[0].av_cabine += CABINE_R; break;
		case 'w':
		case 'W':	tank[0].av_funnel += FUNNEL_R; break;
		case 's':
		case 'S':	tank[0].av_funnel -= FUNNEL_R; break;
		default:
			 Scene::processNormalKeyUp(key, modifiers, mouse);
	}
}

void TankCombat::processSpecialKeyDown(int key, int modifiers, const Point & mouse)
{
	switch(key)
	{
		// change view settings
		case GLUT_KEY_F1:	vx0 = 0; vy0 = 0; vx1 = 1; vy1 = 1;	break;
		case GLUT_KEY_F2:	vx0 = 1; vy0 = 0; vx1 = 1; vy1 = 1;	break;
		case GLUT_KEY_F3:	vx0 = 0; vy0 = 1; vx1 = 1; vy1 = 1;	break;
		case GLUT_KEY_F4:	vx0 = 1; vy0 = 1; vx1 = 1; vy1 = 1;	break;
		case GLUT_KEY_F5:	vx0 = 0; vy0 = 0; vx1 = 2; vy1 = 2;	break;
		
		// control tank
		case GLUT_KEY_LEFT:  	tank[0].turn += 1; 		break;
		case GLUT_KEY_RIGHT:    tank[0].turn -= 1; 		break;
		case GLUT_KEY_UP:       tank[0].drive += 1;		break;
		case GLUT_KEY_DOWN:     tank[0].drive -= 1;		break;

		default:
			 Scene::processSpecialKeyDown(key, modifiers, mouse);
	}
}

void TankCombat::processSpecialKeyUp(int key, int modifiers, const Point & mouse)
{
	switch(key)
	{
		case GLUT_KEY_LEFT:  	tank[0].turn -= 1; 		break;
		case GLUT_KEY_RIGHT:    tank[0].turn += 1; 		break;
		case GLUT_KEY_UP:       tank[0].drive -= 1;		break;
		case GLUT_KEY_DOWN:     tank[0].drive += 1;		break;

		default:
			 Scene::processSpecialKeyUp(key, modifiers, mouse);
	}
}





void TankCombat::drawScene(const Matrix<GLfloat> & world2view, const Frustum & f, bool isTank) const
{
	grid->setDrawBounds(!isTank);

	// we are in view(camera) coordinates, but we want to specify
	// stuff in world coordinates now
	OpenGL::PushMatrix();
	OpenGL::MultMatrix(world2view);

	// draw lamp 0 in world coordinates, (as opposed to model coordinates!)

	lamps[0]->set(true);
//	lamps[0]->draw();

	// the rest is in model coordinates

	OpenGL::PushMatrix();
	OpenGL::MultMatrix(model2world);

	// correct frustum for model coordinates
	Frustum f2(model2world, f.getM());

    // enable the other lamps in scene(model) coordinates
/*	for (unsigned int i = 1; i < lamps.size(); i++)
	{
		lamps[i]->set(true);
		lamps[i]->draw();
	}
*/

	// tank 0 has a beam light attached to its tail
	OpenGL::PushMatrix();
	OpenGL::MultMatrix(tank[0].getMatrix()); // to tank1's coordinate frame
	lamps[1]->set(true);
	OpenGL::PopMatrix();

	// enable all scene lights

	for (unsigned int i = 0; i < objects.size(); i++)
	{
			objects[i]->setLights(true);
	}

	// draw all normal stuff
	for (unsigned int i = 0; i < objects.size(); i++)
	{
			objects[i]->draw(f2);
	}

	// now transparencies
    OpenGL::DepthMask(GL_FALSE); 	// no z buffer writes
	OpenGL::Enable(GL_BLEND);	// alpha blending
	OpenGL::BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		for (unsigned int i = 0; i < objects.size(); i++)
		{
			objects[i]->drawTransparent(f2);
		}
	OpenGL::Disable(GL_BLEND);
	OpenGL::DepthMask(GL_TRUE);

	if (!isTank) f2.draw();
	OpenGL::PopMatrix(); // back to world coordinates
    
	for (unsigned int i = 0; i < objects.size(); i++)
	{
			objects[i]->setLights(false);
	}

	OpenGL::PopMatrix();
}


void TankCombat::draw(int w, int h) const
{
	Scene::draw(w, h);
	if ((w == 0) || (h == 0)) return;

	GLdouble aspect = (GLdouble)w/(GLdouble)h;

	// clear the whole window
	OpenGL::ClearColor(0,0,0,0);
	OpenGL::Clear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// set important OpenGL parameters
	OpenGL::Enable(GL_DEPTH_TEST);
	OpenGL::Enable(GL_CULL_FACE);

	OpenGL::CullFace(GL_BACK);
	OpenGL::Enable(GL_LIGHTING);
	
	double tankdepth = 1<<8;
	
	// set tank (short-sighted) projection matrix
	OpenGL::MatrixMode(GL_PROJECTION);
	OpenGL::LoadIdentity();
	OpenGL::Perspective(45, aspect, 1, tankdepth);

	// frustum goes from screen to world coordinates for our 'tank' view
	Frustum f(world2view[1][1], OpenGL::GetProjectionMatrix());
	

	for (unsigned int y = 0; y < vy1; y++)
	{
		for (unsigned int x = 0; x < vx1; x++)
		{
			bool isTank = vx0+x ==1 && vy0+y == 1;
			
			// set projection matrix
			OpenGL::MatrixMode(GL_PROJECTION);
			OpenGL::LoadIdentity();
			OpenGL::Perspective(45, aspect, 1, isTank ? tankdepth : d+fieldsize);

			// model transformation matrix
			OpenGL::MatrixMode(GL_MODELVIEW);
			OpenGL::LoadIdentity();
			
			
			// note that Y runs upward in OpenGL Viewport!
    		OpenGL::Viewport((x*w)/vx1, ((vy1-y-1)*h)/vy1, w/vx1, h/vy1);
			drawScene(world2view[vy0+y][vx0+x], f, isTank);
		}
	}
}

	
	
	
	


// Rotate the model
void TankCombat::advance(double totaltime, double dt) 
{
	Scene::advance(totaltime, dt);

     //     advance all objects in the scene by the specified amount of time
	for (unsigned int i = 0;   i < objects.size(); i++)
	{
		objects[i]->advance(totaltime, dt);
	}

    // then calculate all pairwise interaction, using double dispatch
    for (unsigned int i = 0;   i < objects.size(); i++)
    {
	    for (unsigned int j = i+1; j < objects.size(); j++)
    	{
			objects[i]->interactWithObject(*objects[j]);
		}
	}


	vector<WorldObject *> oldobjects(objects);
	objects = vector<WorldObject *>();
	
	// finally recollect the objects, that is add new ones and
	// delete old ones that are no longer necessary
	for(unsigned int i = 0; i < oldobjects.size(); i++)
	{
			WorldObject * wo = oldobjects[i];
			if (wo->recollect(objects)) objects.push_back(wo);
			else						delete wo;
	}

	// rotate the world 10 degrees every second
   model2world = HomMatrix<GLfloat>::rotation( g2r(10*totaltime), Vector<GLfloat>(0.0, 1.0, 0.0));

   // follow tank 0 in right-bottom view
   world2view[1][1]   =
    	  HomMatrix<GLfloat>::translation(Vector<GLfloat>(0.0, 0.0, -10)) 		// back up
        * HomMatrix<GLfloat>::rotation(g2r(20), Vector<GLfloat>(1.0, 0.0, 0.0)) 	// tilt slightly
        * HomMatrix<GLfloat>::rotation(g2r(180), Vector<GLfloat>(0.0, 1.0, 0.0)) 	// mirror
        * tank[0].getMatrix().inv() 	// take us to the model from the world
        * model2world.inv(); 			// undo rotation (it sucked anyway)
}





