#include "GL/OpenGL.h"
#include "GL/HomLinAlg.h"
#include "GL/Material.h"

#include "TankModel.h"
#include "array.h"
#include "utils.h"
#include "Grid.h"




Material TankModel::TankMaterial[]    =
{
	Material(
	 Vector<GLfloat>(0.3, 0.1, 0.1, 1),
	 Vector<GLfloat>(0.5, 0.0, 0.0, 1),
	 Vector<GLfloat>(0.0, 0.0, 0.1, 1),
	 Vector<GLfloat>(0, 0, 0, 0),
	 1),
	Material(
	 Vector<GLfloat>(0.0, 0.0, 0.0, 1),
	 Vector<GLfloat>(0.0, 1.0, 0.0, 1),
	 Vector<GLfloat>(1.0, 1.0, 1.0, 1),
	 Vector<GLfloat>(0, 0, 0, 0),
	 1),
	Material(
	 Vector<GLfloat>(0.0, 0.0, 0.0, 1),
	 Vector<GLfloat>(0.0, 0.0, 1.0, 1),
	 Vector<GLfloat>(1.0, 1.0, 1.0, 1),
	 Vector<GLfloat>(0, 0, 0, 0),
	 1),	
	 
};


Vector<GLfloat> TankModel::vs[] =
	{
		Vector<GLfloat>(-0.8, -1, -1.4),
		Vector<GLfloat>(-0.8, -1,  1.1),
		Vector<GLfloat>( 0.8, -1,  1.1),
		Vector<GLfloat>( 0.8, -1, -1.4),
		Vector<GLfloat>(-1,  -0.2, -2),
		Vector<GLfloat>(-1,  -0.2,  2),
		Vector<GLfloat>( 1,  -0.2,  2),
		Vector<GLfloat>( 1,  -0.2, -2),

		Vector<GLfloat>(-0.5, 0.0, -0.6),
		Vector<GLfloat>(-0.5, 0.0,  0.9),
		Vector<GLfloat>( 0.5, 0.0,  0.9),
		Vector<GLfloat>( 0.5, 0.0, -0.6),
		Vector<GLfloat>(-0.4, 0.5, -0.5),
		Vector<GLfloat>(-0.4, 0.5,  0.5),
		Vector<GLfloat>( 0.4, 0.5,  0.5),
		Vector<GLfloat>( 0.4, 0.5, -0.5),

		Vector<GLfloat>(-0.1, 0.0, -0.1),
		Vector<GLfloat>(-0.1, 0.0,  0.1),
		Vector<GLfloat>( 0.1, 0.0,  0.1),
		Vector<GLfloat>( 0.1, 0.0, -0.1),
		Vector<GLfloat>(-0.1, 3.0, -0.1),
		Vector<GLfloat>(-0.1, 3.0,  0.1),
		Vector<GLfloat>( 0.1, 3.0,  0.1),
		Vector<GLfloat>( 0.1, 3.0, -0.1),
	};

Vector<GLfloat> TankModel::ns[] =
	{
		Vector<GLfloat>(-1, -1, -1).normalize(),
		Vector<GLfloat>(-1, -1, +1).normalize(),
		Vector<GLfloat>(+1, -1, +1).normalize(),
		Vector<GLfloat>(+1, -1, -1).normalize(),
		Vector<GLfloat>(-1, +1, -1).normalize(),
		Vector<GLfloat>(-1, +1, +1).normalize(),
		Vector<GLfloat>(+1, +1, +1).normalize(),
		Vector<GLfloat>(+1, +1, -1).normalize(),

		Vector<GLfloat>(-1, 0, -1).normalize(),
		Vector<GLfloat>(-1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, -1).normalize(),
		Vector<GLfloat>(-1, +1, -1).normalize(),
		Vector<GLfloat>(-1, +1, +1).normalize(),
		Vector<GLfloat>(+1, +1, +1).normalize(),
		Vector<GLfloat>(+1, +1, -1).normalize(),

		Vector<GLfloat>(-1, 0, -1).normalize(),
		Vector<GLfloat>(-1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, -1).normalize(),
		Vector<GLfloat>(-1, 0, -1).normalize(),
		Vector<GLfloat>(-1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, +1).normalize(),
		Vector<GLfloat>(+1, 0, -1).normalize(),
								
	};
 


TankModel::TankModel()
	: 	nvertices(sizein(vs)),
		vertexbuffer(new GLfloat[3 * nvertices]),
		normalbuffer(new GLfloat[3 * nvertices])
	
{
	for (unsigned int i = 0; i < sizein(vs); i++)
	for (unsigned int j = 0; j < 3; j++)
	{
		vertexbuffer[i*3+j] = vs[i](j);  
	}


	for (unsigned int i = 0; i < sizein(ns); i++)
	for (unsigned int j = 0; j < 3; j++)
	{
		normalbuffer[i*3+j] = ns[i](j);
	}

	indexbuffer = new_array<GLuint>(3*6, 4);
	indexbuffer[0][0] = 0;
	indexbuffer[0][1] = 3;
	indexbuffer[0][2] = 1;
	indexbuffer[0][3] = 2;

	indexbuffer[1][0] = 3;
	indexbuffer[1][1] = 7;
	indexbuffer[1][2] = 2;
	indexbuffer[1][3] = 6;

	indexbuffer[2][0] = 7;
	indexbuffer[2][1] = 4;
	indexbuffer[2][2] = 6;
	indexbuffer[2][3] = 5;

	indexbuffer[3][0] = 4;
	indexbuffer[3][1] = 0;
	indexbuffer[3][2] = 5;
	indexbuffer[3][3] = 1;

	indexbuffer[4][0] = 5;
	indexbuffer[4][1] = 1;
	indexbuffer[4][2] = 6;
	indexbuffer[4][3] = 2;

	indexbuffer[5][0] = 7;
	indexbuffer[5][1] = 3;
	indexbuffer[5][2] = 4;
	indexbuffer[5][3] = 0;

	indexbuffer[6][0] = 0+1*8;
	indexbuffer[6][1] = 3+1*8;
	indexbuffer[6][2] = 1+1*8;
	indexbuffer[6][3] = 2+8;

	indexbuffer[7][0] = 3+8;
	indexbuffer[7][1] = 7+8;
	indexbuffer[7][2] = 2+8;
	indexbuffer[7][3] = 6+8;

	indexbuffer[8][0] = 7+8;
	indexbuffer[8][1] = 4+8;
	indexbuffer[8][2] = 6+8;
	indexbuffer[8][3] = 5+8;

	indexbuffer[9][0] = 4+8;
	indexbuffer[9][1] = 0+8;
	indexbuffer[9][2] = 5+8;
	indexbuffer[9][3] = 1+8;

	indexbuffer[10][0] = 5+8;
	indexbuffer[10][1] = 1+8;
	indexbuffer[10][2] = 6+8;
	indexbuffer[10][3] = 2+8;

	indexbuffer[11][0] = 7+8;
	indexbuffer[11][1] = 3+8;
	indexbuffer[11][2] = 4+8;
	indexbuffer[11][3] = 0+8;


	indexbuffer[2*6+0][0] = 0+2*8;
	indexbuffer[2*6+0][1] = 3+2*8;
	indexbuffer[2*6+0][2] = 1+2*8;
	indexbuffer[2*6+0][3] = 2+2*8;

	indexbuffer[2*6+1][0] = 3+2*8;
	indexbuffer[2*6+1][1] = 7+2*8;
	indexbuffer[2*6+1][2] = 2+2*8;
	indexbuffer[2*6+1][3] = 6+2*8;

	indexbuffer[2*6+2][0] = 7+2*8;
	indexbuffer[2*6+2][1] = 4+2*8;
	indexbuffer[2*6+2][2] = 6+2*8;
	indexbuffer[2*6+2][3] = 5+2*8;

	indexbuffer[2*6+3][0] = 4+2*8;
	indexbuffer[2*6+3][1] = 0+2*8;
	indexbuffer[2*6+3][2] = 5+2*8;
	indexbuffer[2*6+3][3] = 1+2*8;

	indexbuffer[2*6+4][0] = 5+2*8;
	indexbuffer[2*6+4][1] = 1+2*8;
	indexbuffer[2*6+4][2] = 6+2*8;
	indexbuffer[2*6+4][3] = 2+2*8;

	indexbuffer[2*6+5][0] = 7+2*8;
	indexbuffer[2*6+5][1] = 3+2*8;
	indexbuffer[2*6+5][2] = 4+2*8;
	indexbuffer[2*6+5][3] = 0+2*8;
}

	
TankModel::~TankModel()
{
}


void TankModel::draw(double a_cabine, double a_funnel) const
{
	OpenGL::PushMatrix();

	OpenGL::VertexPointer(3, GL_FLOAT, 0, vertexbuffer);
	OpenGL::NormalPointer(   GL_FLOAT, 0, normalbuffer);
	OpenGL::EnableClientState(GL_VERTEX_ARRAY);
	OpenGL::EnableClientState(GL_NORMAL_ARRAY);

	TankMaterial[0].set();
	for (unsigned int x = 0; x < 6; x++)
	{
		OpenGL::DrawElements(GL_TRIANGLE_STRIP, 2+2*1, GL_UNSIGNED_INT, indexbuffer[x]);
	}
//    drawVertexNormals(1, 0, 8);


	OpenGL::MultMatrix(HomMatrix<GLfloat>::translation(Vector<GLfloat>(0, -0.2, -0.3)));
	OpenGL::MultMatrix(HomMatrix<GLfloat>::rotation(a_cabine, Vector<GLfloat>(0, 1, 0)));

	TankMaterial[1].set();
	for (unsigned int x = 6; x < 12; x++)
	{
		OpenGL::DrawElements(GL_TRIANGLE_STRIP, 2+2*1, GL_UNSIGNED_INT, indexbuffer[x]);
	}
//	drawVertexNormals(1, 8, 16);


	OpenGL::MultMatrix(HomMatrix<GLfloat>::translation(Vector<GLfloat>(0, 0, 0.2)));
	OpenGL::MultMatrix(HomMatrix<GLfloat>::rotation(a_funnel, Vector<GLfloat>(1, 0, 0)));

	TankMaterial[2].set();
	for (unsigned int x = 12; x < 18; x++)
	{
		OpenGL::DrawElements(GL_TRIANGLE_STRIP, 2+2*1, GL_UNSIGNED_INT, indexbuffer[x]);
	}
//	drawVertexNormals(1, 16, 24);
	

	OpenGL::DisableClientState(GL_VERTEX_ARRAY);
	OpenGL::DisableClientState(GL_NORMAL_ARRAY);

	OpenGL::PopMatrix();
}

